@@ -21,6 +21,7 @@ from __future__ import annotations
 import contextlib
 import copy
 import json
+import logging
 import os
 import select
 import shlex
@@ -35,15 +36,14 @@ from packaging.version import Version
 from airflow.exceptions import AirflowConfigException, AirflowException
 from airflow.hooks.base import BaseHook
 from airflow.providers.google.go_module_utils import init_module, install_dependencies
-from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.python_virtualenv import prepare_virtualenv
 
 
 class BeamRunnerType:
     """
     Helper class for listing runner types.
-    For more information about runners see:
-    https://beam.apache.org/documentation/
+
+    For more information about runners see: https://beam.apache.org/documentation/
     """
 
     DataflowRunner = "DataflowRunner"
@@ -58,7 +58,7 @@ class BeamRunnerType:
 
 def beam_options_to_args(options: dict) -> list[str]:
     """
-    Returns a formatted pipeline options from a dictionary of arguments
+    Returns a formatted pipeline options from a dictionary of arguments.
 
     The logic of this method should be compatible with Apache Beam:
     https://github.com/apache/beam/blob/b56740f0e8cd80c2873412847d0b336837429fb9/sdks/python/
@@ -81,81 +81,85 @@ def beam_options_to_args(options: dict) -> list[str]:
     return args
 
 
-class BeamCommandRunner(LoggingMixin):
+def process_fd(
+    proc,
+    fd,
+    log: logging.Logger,
+    process_line_callback: Callable[[str], None] | None = None,
+):
     """
-    Class responsible for running pipeline command in subprocess
+    Prints output to logs.
+
+    :param proc: subprocess.
+    :param fd: File descriptor.
+    :param process_line_callback: Optional callback which can be used to process
+        stdout and stderr to detect job id.
+    :param log: logger.
+    """
+    if fd not in (proc.stdout, proc.stderr):
+        raise Exception("No data in stderr or in stdout.")
+
+    fd_to_log = {proc.stderr: log.warning, proc.stdout: log.info}
+    func_log = fd_to_log[fd]
+
+    while True:
+        line = fd.readline().decode()
+        if not line:
+            return
+        if process_line_callback:
+            process_line_callback(line)
+        func_log(line.rstrip("\n"))
+
+
+def run_beam_command(
+    cmd: list[str],
+    log: logging.Logger,
+    process_line_callback: Callable[[str], None] | None = None,
+    working_directory: str | None = None,
+) -> None:
+    """
+    Function responsible for running pipeline command in subprocess.
 
     :param cmd: Parts of the command to be run in subprocess
     :param process_line_callback: Optional callback which can be used to process
         stdout and stderr to detect job id
     :param working_directory: Working directory
+    :param log: logger.
     """
+    log.info("Running command: %s", " ".join(shlex.quote(c) for c in cmd))
 
-    def __init__(
-        self,
-        cmd: list[str],
-        process_line_callback: Callable[[str], None] | None = None,
-        working_directory: str | None = None,
-    ) -> None:
-        super().__init__()
-        self.log.info("Running command: %s", " ".join(shlex.quote(c) for c in cmd))
-        self.process_line_callback = process_line_callback
-        self.job_id: str | None = None
+    proc = subprocess.Popen(
+        cmd,
+        cwd=working_directory,
+        shell=False,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        close_fds=True,
+    )
+    # Waits for Apache Beam pipeline to complete.
+    log.info("Start waiting for Apache Beam process to complete.")
+    reads = [proc.stderr, proc.stdout]
+    while True:
+        # Wait for at least one available fd.
+        readable_fds, _, _ = select.select(reads, [], [], 5)
+        if readable_fds is None:
+            log.info("Waiting for Apache Beam process to complete.")
+            continue
 
-        self._proc = subprocess.Popen(
-            cmd,
-            cwd=working_directory,
-            shell=False,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            close_fds=True,
-        )
+        for readable_fd in readable_fds:
+            process_fd(proc, readable_fd, log, process_line_callback)
 
-    def _process_fd(self, fd):
-        """
-        Prints output to logs.
+        if proc.poll() is not None:
+            break
 
-        :param fd: File descriptor.
-        """
-        if fd not in (self._proc.stdout, self._proc.stderr):
-            raise Exception("No data in stderr or in stdout.")
+    # Corner case: check if more output was created between the last read and the process termination
+    for readable_fd in reads:
+        process_fd(proc, readable_fd, log, process_line_callback)
 
-        fd_to_log = {self._proc.stderr: self.log.warning, self._proc.stdout: self.log.info}
-        func_log = fd_to_log[fd]
+    log.info("Process exited with return code: %s", proc.returncode)
 
-        while True:
-            line = fd.readline().decode()
-            if not line:
-                return
-            if self.process_line_callback:
-                self.process_line_callback(line)
-            func_log(line.rstrip("\n"))
-
-    def wait_for_done(self) -> None:
-        """Waits for Apache Beam pipeline to complete."""
-        self.log.info("Start waiting for Apache Beam process to complete.")
-        reads = [self._proc.stderr, self._proc.stdout]
-        while True:
-            # Wait for at least one available fd.
-            readable_fds, _, _ = select.select(reads, [], [], 5)
-            if readable_fds is None:
-                self.log.info("Waiting for Apache Beam process to complete.")
-                continue
-
-            for readable_fd in readable_fds:
-                self._process_fd(readable_fd)
-
-            if self._proc.poll() is not None:
-                break
-
-        # Corner case: check if more output was created between the last read and the process termination
-        for readable_fd in reads:
-            self._process_fd(readable_fd)
-
-        self.log.info("Process exited with return code: %s", self._proc.returncode)
-
-        if self._proc.returncode != 0:
-            raise AirflowException(f"Apache Beam process failed with return code {self._proc.returncode}")
+    if proc.returncode != 0:
+        raise AirflowException(f"Apache Beam process failed with return code {proc.returncode}")
 
 
 class BeamHook(BaseHook):
@@ -187,12 +191,12 @@ class BeamHook(BaseHook):
         ]
         if variables:
             cmd.extend(beam_options_to_args(variables))
-        cmd_runner = BeamCommandRunner(
+        run_beam_command(
             cmd=cmd,
             process_line_callback=process_line_callback,
             working_directory=working_directory,
+            log=self.log,
         )
-        cmd_runner.wait_for_done()
 
     def start_python_pipeline(
         self,
